#!/usr/bin/env python
# -*- coding: utf-8 -*-

import re
import sys
import traceback


if sys.version_info < (3,):
    import yaml_2 as yaml
else:
    import yaml_3 as yaml



class MongoParser(object):
    def __init__(self):
        self.conf_file = ''
        self.type = None
        self._cnf_dict = {}
        self.extra_cmdline = []

    def get_value(self, cnf_dict, key_list):
        """
        get value from recursive dict:
        return value:    the value we want to get if  key is not exist None will be return
        key_list:        it is a key list or just a key
        """
        if not isinstance(cnf_dict, dict):
            raise
        if isinstance(key_list, list):
            for i in range(len(key_list)):
                value = cnf_dict.get(key_list[i])
                if isinstance(value, dict):
                    son_dict = value
                    son_key_list = key_list[(i + 1):]
                    final_value = self.get_value(son_dict, son_key_list)
                    return final_value

                elif isinstance(value, str) or isinstance(value, int) or isinstance(value, bool):

                    if i == len(key_list) - 1:
                        return value
                    else:
                        pass
                else:
                    return None

        if isinstance(key_list, str):
            return cnf_dict.get(key_list)

    def recursion_dict(self, tmp_list, cnf_dict):
        """create recursive dictionary"""
        if len(tmp_list) < 2:
            raise Exception('%s is not a key value type !' % tmp_list)
        if len(tmp_list) == 2:
            return cnf_dict.update({tmp_list[0]: tmp_list[1]})
        else:
            son_cnf_dict = {}
            if tmp_list[0] in cnf_dict:
                son_cnf_dict = cnf_dict[tmp_list[0]]
            
            self.recursion_dict(tmp_list[1:], son_cnf_dict)
            cnf_dict.update({tmp_list[0]: son_cnf_dict})

    def generate_cnf_dict(self, x2):
        """generate a cnf_dict"""
        keyvalue_list = x2.split('\n')
        keyvalue_list = [x.strip() for x in keyvalue_list if len(x.strip()) != 0]

        cnf_dict = {}
        for keyvalue in keyvalue_list:
            tmp_list = keyvalue.split('=')
            tmp_list = [x.strip() for x in tmp_list]
            self.recursion_dict(tmp_list, cnf_dict)

        return cnf_dict

    def content_filter(self, lines):
        """filter line which is comment or null line"""
        msg_info = []

        for line in lines:
            if line.strip().startswith('#') or 0 == len(line.strip()):
                continue
            m = re.match(r"(?P<parameter>.+)(?P<comment>#.+$)", line)
            if m is not None:
                msg_info.append(''.join([m.group('parameter'), '\n']))
            else:
                msg_info.append(line)

        return '\n'.join(msg_info)

    def get_conf_value(self, yml_key_list, key_list):
        """get config value"""
        cnf_dict = self._cnf_dict
        if self.type == 'str':
            return self.get_value(cnf_dict, key_list)
        elif self.type == 'dict':
            return self.get_value(cnf_dict, yml_key_list)
        else:
            return None

    def get_cmd_value(self, cmd_params):
        """get cmd params"""
        extra_cmdline = self.extra_cmdline
        ret_cmd_value = []
        ret_cmd_param = []
        
        if isinstance(cmd_params, str):
            tmp = cmd_params
            cmd_params = []
            cmd_params.append(tmp)
            
        if isinstance(cmd_params, list):
            pass
        else:
            raise Exception('%s type is %s, but we need str or list!' % (cmd_params, type(cmd_params)))
            
        num = 0
        for cmd_param in extra_cmdline:
            num = num + 1
            if cmd_param in cmd_params:
                tmp_cmd_value = ''
                ret_cmd_param.append(cmd_param)
                if num == len(extra_cmdline):
                    tmp_cmd_value = ''
                else:
                    tmp_cmd_value = '' if extra_cmdline[num].startswith('-') else extra_cmdline[num]
                ret_cmd_value.append(tmp_cmd_value)
            elif '=' in cmd_param:
                tmp_cmd_value = ''
                tmp_cmd = cmd_param.split('=', 1)[0]
                if tmp_cmd in cmd_params:
                    ret_cmd_param.append(tmp_cmd)
                    tmp_cmd_value = cmd_param.split('=', 1)[1]
                    ret_cmd_value.append(tmp_cmd_value)

        if len(ret_cmd_param) > 1:
            return ret_cmd_param, ret_cmd_value
        return ''.join(ret_cmd_param), ''.join(ret_cmd_value)

    def set_conf_file(self, value):
        """set config file"""
        self.conf_file = value

    def get_conf_file(self):
        """get config file"""
        return self.conf_file

    def set_extra_cmdline(self, value):
        """set extra cmd"""
        if isinstance(value, list):
            self.extra_cmdline = value
        else:
            self.extra_cmdline = value.split()

    def get_extra_cmdline(self):
        """get extra cmd"""
        return self.extra_cmdline

    def run_parser(self):
        """parser config file"""
        self.conf_file = self.get_conf_file()
        if self.conf_file != "":
            with open(self.conf_file, 'r') as file_var:
                lines = file_var.readlines()
                context = self.content_filter(lines)
                context_value = yaml.load(context)

                if isinstance(context_value, str):  #key value
                    self.type = 'str'
                    self._cnf_dict = self.generate_cnf_dict(context_value)
                elif isinstance(context_value, dict):  #yaml
                    self.type = 'dict'
                    self._cnf_dict = context_value
                else:
                    raise Exception('%s is not support!' % type(context_value))
